@@ -5,6 +5,7 @@ import pickle  # Only for backward compatibility
 import posixpath
 import re
 import tempfile
+import base64
 from collections import defaultdict
 from typing import Any
 
@@ -19,6 +20,7 @@ from .filesystem import FileSystem, clear_cache
 
 # from .helpers.metadata import collect_parquet_metadata  # , remove_from_metadata
 from .helpers.misc import get_partitions_from_path, run_parallel, unify_schemas_pl
+import brotli
 from .schema import (
     convert_large_types_to_normal,  # unify_schemas
     repair_schema,
@@ -26,12 +28,22 @@ from .schema import (
 
 
 def serialize_metadata(metadata: dict[str, pq.FileMetaData]) -> dict[str, Any]:
-    """Safely serialize metadata to JSON-compatible format."""
-    result = {}
+    """Safely serialize metadata to a JSON-compatible format.
+
+    We write each ``FileMetaData`` to an in-memory buffer using
+    ``write_metadata_file`` and base64-encode the resulting bytes for JSON.
+    """
+    result: dict[str, Any] = {}
     for path, meta in metadata.items():
-        # Convert FileMetaData to serializable format
+        # Write FileMetaData to an in-memory buffer
+        sink = pa.BufferOutputStream()
+        meta.write_metadata_file(sink)
+        buf = sink.getvalue()  # pyarrow.Buffer
+        raw: bytes = buf.to_pybytes()
+        # Base64 encode to make it JSON serializable
+        b64 = base64.b64encode(raw).decode("ascii")
         result[path] = {
-            "serialized_metadata": meta.serialize(),
+            "serialized_metadata_b64": b64,
             "num_rows": meta.num_rows,
             "num_columns": len(meta.schema),
             "created_by": meta.created_by,
@@ -41,12 +53,32 @@ def serialize_metadata(metadata: dict[str, pq.FileMetaData]) -> dict[str, Any]:
 
 
 def deserialize_metadata(data: dict[str, Any]) -> dict[str, pq.FileMetaData]:
-    """Safely deserialize metadata from JSON-compatible format."""
-    result = {}
+    """Safely deserialize metadata from JSON-compatible format.
+
+    Expects base64-encoded bytes under the key ``serialized_metadata_b64``.
+    Falls back to ``serialized_metadata`` if present and already a base64 string.
+    """
+    result: dict[str, pq.FileMetaData] = {}
     for path, meta_data in data.items():
         # Reconstruct FileMetaData from serialized data
-        buf = pa.py_buffer(meta_data["serialized_metadata"])
-        result[path] = pq.read_metadata(buf)
+        if "serialized_metadata_b64" in meta_data:
+            raw_bytes = base64.b64decode(meta_data["serialized_metadata_b64"])  # type: ignore[arg-type]
+        else:
+            # Backward-compatibility: if an older key is present and contains a
+            # base64 string, try decoding it; if it's bytes-like, use directly.
+            sm = meta_data.get("serialized_metadata")
+            if isinstance(sm, str):
+                raw_bytes = base64.b64decode(sm)
+            elif isinstance(sm, (bytes, bytearray)):
+                raw_bytes = bytes(sm)
+            else:
+                raise ValueError(
+                    "Serialized metadata missing or in unknown format for path: "
+                    + str(path)
+                )
+
+        reader = pa.BufferReader(raw_bytes)
+        result[path] = pq.read_metadata(reader)
     return result
 
 
@@ -216,8 +248,16 @@ class ParquetDatasetMetadata:
         if self.has_file_metadata_file:
             # First try to open as text (JSON)
             try:
-                with self._filesystem.open(self._file_metadata_file, "r") as f:
-                    data = json.load(f)
+                with self._filesystem.open(self._file_metadata_file, "rb") as f:
+                    compressed_data = f.read()
+                    try:
+                        # Try to decompress with brotli first
+                        decompressed_data = brotli.decompress(compressed_data)
+                        data = json.loads(decompressed_data.decode('utf-8'))
+                    except (brotli.error, UnicodeDecodeError):
+                        # Fall back to uncompressed JSON
+                        f.seek(0)
+                        data = json.loads(compressed_data.decode('utf-8'))
                     return deserialize_metadata(data)
             except (json.JSONDecodeError, UnicodeDecodeError):
                 # Fall back to binary pickle format for backward compatibility
@@ -319,8 +359,13 @@ class ParquetDatasetMetadata:
         Save file metadata to a specified file.
         """
         # Use JSON format instead of pickle for security
-        with self._filesystem.open(self._file_metadata_file, "w") as f:
-            json.dump(serialize_metadata(self._file_metadata), f, indent=2)
+        # Compress the serialized data using brotli
+        
+        serialized_data = json.dumps(serialize_metadata(self._file_metadata), indent=2)
+        compressed_data = brotli.compress(serialized_data.encode('utf-8'), quality=5)
+        
+        with self._filesystem.open(self._file_metadata_file, "wb") as f:
+            f.write(compressed_data)
 
     def update_file_metadata(
         self, files: list[str] | None = None, verbose: bool = False, **kwargs
